#!/usr/bin/env python

import os,sys
import logging
import imp
import math
import collections
import numpy

tmp_path = "%s/git/versuchung/src"% os.environ["HOME"]
if os.path.exists(tmp_path):
    sys.path.append(tmp_path)

from versuchung.experiment import Experiment
from versuchung.types import String, Bool,List
from versuchung.files import File, Directory
from versuchung.archives import GitArchive
from versuchung.execute import shell
from versuchung.tex import DatarefDict

class AnalysisQuality(Experiment):
    inputs = {
        "coredos": GitArchive("/proj/i4gerrit/review_site/git/coredos.git"),
    }
    outputs = {"dicts": Directory("dicts"),
               "stats": File("statistics.py")}

    def run(self):
        logging.info("Cloning coredos...")
        with self.coredos as coredos_path:
            self.stats.copy_contents("generator/statistics.py")

            os.mkdir("build");
            logging.info("Configuring...")

            # Activate coptermock-no-annotation
            shell("echo 'DOSEK_BINARY(NAME bench-coptermock-no-annotation SYSTEM_DESC system-no-annotation.oil FAIL dosek.cc)' >> app/benchmark/coptermock/CMakeLists.txt")
            shell("cd build; ../new_build_env.py -a i386 -c --generator-args='-fstate-flow -fsse -fgcfg'")

            logging.info("Building...")
            shell("cd build; make  bench-coptermock bench-coptermock-no-annotation")


            (stats_files, _) = shell("find build -name stats.dict.py")

            for filename in stats_files:
                name = filename.split("/")[-2]
                logging.info("Gathering for %s", name)
                f = self.dicts.new_file(name)
                f.copy_contents(filename)


class AnalysisQualityCombine(Experiment):
    inputs = {"gather": AnalysisQuality("AnalysisQuality") }
    outputs = {"dref" : DatarefDict() }

    def fmtn(self,name):
        return name.replace("bcc1_", "").replace("_", "-")

    def key(self, keys, value):
        self.dref["/".join(["analysis"] + keys)] = value

    def work_on_benchmark(self, name, stats):
        def S(keys, value):
            self.key([self.fmtn(name)] + keys, value)
        sse = stats.find_one("SymbolicSystemExecution")
        ssf = stats.find_one("SystemStateFlow")
        gcfg = stats.find_one("ConstructGlobalCFG")

        S(['abb count'], gcfg['abb-count'])
        syscalls = 0
        for x in stats.find_all('AtomicBasicBlock').values():
            if '/' in x["_name"]:
                # If name is of the form: ABB[0-9]/lalala it is a syscall
                syscalls += 1
        S(['syscall count'], syscalls)

        for k in ("full", "structure", "oil-static", "sse-uncut", "ssf-uncut"):
            S(["edges", k], gcfg["edge-count:"+k])

        S(['sse', 'runtime'], sse['run-time'])
        S(['sse', 'states'], sse['system-states'])
        S(['sse', 'transitions'], sse['state-transitions:sse-uncut'])

        S(['ssf', 'runtime'], ssf['run-time'])


    def run(self):
        # Load the statistics module, that was imported from coredos
        # to read the statstics
        statistics = imp.load_source("statistics", self.gather.stats.path)

        self.matrix = collections.defaultdict(list)
        self.gcfgs = []
        for tmp in self.gather.dicts:
            name = tmp.basename
            stats = statistics.Statistics.load(tmp.path)
            self.work_on_benchmark(name, stats)



if __name__ == "__main__":
    import shutil, sys
    if len(sys.argv) > 1:
        action = sys.argv[1]; del sys.argv[1]
    actions = {"gather": AnalysisQuality,
               "combine": AnalysisQualityCombine,
              }

    if action in actions:
        experiment = actions[action]()
        dirname = experiment(sys.argv)
    else:
        print "No action to be taken, use: %s" % ", ".join(actions.keys())

